from typing import Any, Callable, Tuple, List, Dict, Union
import logging
from datamaxi.api import API
import pandas as pd
from datamaxi.lib.utils import check_required_parameters
from datamaxi.datamaxi.utils import convert_data_to_data_frame
from datamaxi.lib.constants import ASC, DESC


class Dex(API):
    """Client to fetch DEX data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize DEX client.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        super().__init__(api_key, **kwargs)

    def trade(
        self,
        chain: str,
        exchange: str,
        pool: str,
        page: int = 1,
        limit: int = 1000,
        fromDateTime: str = None,
        toDateTime: str = None,
        sort: str = "desc",
        pandas: bool = True,
    ) -> Union[Tuple[Dict, Callable], Tuple[pd.DataFrame, Callable]]:
        """Fetch DEX trade data

        `GET /api/v1/dex/trade`

        <https://docs.datamaxiplus.com/rest/dex/trade>

        Args:
            chain (str): Chain name
            exchange (str): Exchange name
            pool (str): Pool name
            page (int): Page number
            limit (int): Limit of data
            fromDateTime (str): Start date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            toDateTime (str): End date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            sort (str): Sort order
            pandas (bool): Return data as pandas DataFrame

        Returns:
            DEX trade data in pandas DataFrame and next request function
        """
        logging.warning("warning: dex related endpoints are experimental")

        check_required_parameters(
            [
                [chain, "chain"],
                [exchange, "exchange"],
                [pool, "pool"],
            ]
        )
        if page < 1:
            raise ValueError("page must be greater than 0")

        if limit < 1:
            raise ValueError("limit must be greater than 0")

        if fromDateTime is not None and toDateTime is not None:
            raise ValueError(
                "fromDateTime and toDateTime cannot be set at the same time"
            )

        if sort not in [ASC, DESC]:
            raise ValueError("sort must be either asc or desc")

        params = {
            "chain": chain,
            "exchange": exchange,
            "pool": pool,
            "page": page,
            "limit": limit,
            "from": fromDateTime,
            "to": toDateTime,
            "sort": sort,
        }

        res = self.query("/api/v1/dex/trade", params)
        if res["data"] is None or len(res["data"]) == 0:
            raise ValueError("no data found")

        def next_request():
            return self.get(
                chain,
                exchange,
                pool,
                page + 1,
                limit,
                fromDateTime,
                toDateTime,
                sort,
                pandas,
            )

        if pandas:
            df = convert_data_to_data_frame(res["data"], ["b", "bq", "qq", "p"])
            return df, next_request
        else:
            return res, next_request

    def candle(
        self,
        chain: str,
        exchange: str,
        pool: str,
        interval: str = "1d",
        page: int = 1,
        limit: int = 1000,
        fromDateTime: str = None,
        toDateTime: str = None,
        sort: str = "desc",
        pandas: bool = True,
    ) -> Union[Tuple[Dict, Callable], Tuple[pd.DataFrame, Callable]]:
        """Fetch DEX candle data

        `GET /api/v1/dex/candle`

        <https://docs.datamaxiplus.com/rest/dex/candle>

        Args:
            chain (str): Chain name
            exchange (str): Exchange name
            pool (str): Pool name
            interval (str): Candle interval
            page (int): Page number
            limit (int): Limit of data
            fromDateTime (str): Start date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            toDateTime (str): End date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            sort (str): Sort order
            pandas (bool): Return data as pandas DataFrame

        Returns:
            DEX candle data in pandas DataFrame and next request function
        """
        logging.warning("warning: dex related endpoints are experimental")

        check_required_parameters(
            [
                [chain, "chain"],
                [exchange, "exchange"],
                [pool, "pool"],
                [interval, "interval"],
            ]
        )

        if page < 1:
            raise ValueError("page must be greater than 0")

        if limit < 1:
            raise ValueError("limit must be greater than 0")

        if fromDateTime is not None and toDateTime is not None:
            raise ValueError(
                "fromDateTime and toDateTime cannot be set at the same time"
            )

        if sort not in [ASC, DESC]:
            raise ValueError("sort must be either asc or desc")

        params = {
            "chain": chain,
            "exchange": exchange,
            "pool": pool,
            "interval": interval,
            "page": page,
            "limit": limit,
            "from": fromDateTime,
            "to": toDateTime,
            "sort": sort,
        }

        res = self.query("/api/v1/dex/candle", params)
        if res["data"] is None or len(res["data"]) == 0:
            raise ValueError("no data found")

        def next_request():
            return self.get(
                chain,
                exchange,
                pool,
                interval,
                page + 1,
                limit,
                fromDateTime,
                toDateTime,
                sort,
                pandas,
            )

        if pandas:
            df = convert_data_to_data_frame(res["data"])
            return df, next_request
        else:
            return res, next_request

    def liquidity(
        self,
        chain: str,
        exchange: str,
        pool: str,
        page: int = 1,
        limit: int = 1000,
        fromDateTime: str = None,
        toDateTime: str = None,
        sort: str = "desc",
        pandas: bool = True,
    ) -> Union[Tuple[Dict, Callable], Tuple[pd.DataFrame, Callable]]:
        """Fetch DEX liquidity data

        `GET /api/v1/dex/liquidity`

        <https://docs.datamaxiplus.com/rest/dex/liquidity>

        Args:
            chain (str): Chain name
            exchange (str): Exchange name
            pool (str): Pool name
            page (int): Page number
            limit (int): Limit of data
            fromDateTime (str): Start date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            toDateTime (str): End date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            sort (str): Sort order
            pandas (bool): Return data as pandas DataFrame

        Returns:
            DEX liquidity data in pandas DataFrame and next request function
        """
        logging.warning("warning: dex related endpoints are experimental")

        check_required_parameters(
            [
                [chain, "chain"],
                [exchange, "exchange"],
                [pool, "pool"],
            ]
        )

        if page < 1:
            raise ValueError("page must be greater than 0")

        if limit < 1:
            raise ValueError("limit must be greater than 0")

        if fromDateTime is not None and toDateTime is not None:
            raise ValueError(
                "fromDateTime and toDateTime cannot be set at the same time"
            )

        if sort not in [ASC, DESC]:
            raise ValueError("sort must be either asc or desc")

        params = {
            "chain": chain,
            "exchange": exchange,
            "pool": pool,
            "page": page,
            "limit": limit,
            "from": fromDateTime,
            "to": toDateTime,
            "sort": sort,
        }

        res = self.query("/api/v1/dex/liquidity", params)
        if res["data"] is None or len(res["data"]) == 0:
            raise ValueError("no data found")

        def next_request():
            return self.get(
                chain,
                exchange,
                pool,
                page + 1,
                limit,
                fromDateTime,
                toDateTime,
                sort,
                pandas,
            )

        if pandas:
            df = convert_data_to_data_frame(
                res["data"],
            )
            return df, next_request
        else:
            return res, next_request

    def chains(self) -> List[str]:
        """Fetch supported chains accepted by
        [datamaxi.Dex.candle](./#datamaxi.datamaxi.Dex.candle),
        [datamaxi.Dex.trade](./#datamaxi.datamaxi.Dex.trade) and
        [datamaxi.Dex.liquidity](./#datamaxi.datamaxi.Dex.liquidity).

        `GET /api/v1/dex/chains`

        <https://docs.datamaxiplus.com/rest/dex/chains>

        Returns:
            List of supported chains
        """
        logging.warning("warning: dex related endpoints are experimental")

        url_path = "/api/v1/dex/chains"
        return self.query(url_path)

    def exchanges(self) -> List[str]:
        """Fetch supported exchanges accepted by
        [datamaxi.Dex.candle](./#datamaxi.datamaxi.Dex.candle),
        [datamaxi.Dex.trade](./#datamaxi.datamaxi.Dex.trade) and
        [datamaxi.Dex.liquidity](./#datamaxi.datamaxi.Dex.liquidity).

        `GET /api/v1/dex/exchanges`

        <https://docs.datamaxiplus.com/rest/dex/exchanges>

        Returns:
            List of supported exchanges
        """
        logging.warning("warning: dex related endpoints are experimental")

        url_path = "/api/v1/dex/exchanges"
        return self.query(url_path)

    def pools(self, exchange: str = None, chain: str = None) -> List[Dict]:
        """Fetch supported pools accepted by
        [datamaxi.Dex.candle](./#datamaxi.datamaxi.Dex.candle),
        [datamaxi.Dex.trade](./#datamaxi.datamaxi.Dex.trade) and
        [datamaxi.Dex.liquidity](./#datamaxi.datamaxi.Dex.liquidity).

        `GET /api/v1/dex/pools`

        <https://docs.datamaxiplus.com/rest/dex/pools>

        Args:
            exchange (str): Exchange name
            chain (str): Chain name (applied to DEX only)

        Returns:
            List of supported pools
        """
        params = {}
        if exchange is not None:
            params["exchange"] = exchange
        if chain is not None:
            params["chain"] = chain

        url_path = "/api/v1/dex/pools"
        return self.query(url_path, params)

    def intervals(self) -> List[str]:
        """Fetch supported intervals accepted by
        [datamaxi.Dex.candle](./#datamaxi.datamaxi.Dex.candle).

        `GET /api/v1/dex/intervals`

        <https://docs.datamaxiplus.com/rest/dex/intervals>

        Returns:
            List of supported intervals
        """
        logging.warning("warning: dex related endpoints are experimental")

        url_path = "/api/v1/dex/intervals"
        return self.query(url_path)



from typing import Any, List, Dict, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameters
from datamaxi.lib.utils import check_required_parameter


class CexWalletStatus(API):
    """Client to fetch transfer status data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize wallet status client.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        super().__init__(api_key, **kwargs)

    def __call__(
        self,
        exchange: str,
        asset: str,
        pandas: bool = True,
    ) -> Union[Dict, pd.DataFrame]:
        """Fetch transfer status data

        `GET /api/v1/wallet-status`

        <https://docs.datamaxiplus.com/rest/cex/wallet-status/data>

        Args:
            exchange (str): Exchange name
            asset (str): Asset name
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Wallet status data
        """
        check_required_parameters(
            [
                [exchange, "exchange"],
                [asset, "asset"],
            ]
        )

        params = {
            "exchange": exchange,
            "asset": asset,
        }

        url_path = "/api/v1/wallet-status"
        res = self.query(url_path, params)
        if pandas:
            df = pd.DataFrame(res)
            df = df.set_index("network")
            return df

        return res

    def exchanges(self) -> List[str]:
        """Fetch supported exchanges accepted by
        [datamaxi.CexWalletStatus.__call__](./#datamaxi.datamaxi.CexWalletStatus.__call__)
        API.

        `GET /api/v1/wallet-status/exchanges`

        <https://docs.datamaxiplus.com/rest/cex/wallet-status/exchanges>

        Returns:
            List of supported exchange
        """
        url_path = "/api/v1/wallet-status/exchanges"
        return self.query(url_path)

    def assets(self, exchange: str) -> List[str]:
        """Fetch supported assets accepted by
        [datamaxi.CexWalletStatus.__call__](./#datamaxi.datamaxi.CexWalletStatus.__call__)
        API.

        `GET /api/v1/wallet-status/assets`

        <https://docs.datamaxiplus.com/rest/cex/wallet-status/assets>

        Args:
            exchange (str): Exchange name

        Returns:
            List of supported assets
        """
        check_required_parameter(exchange, "exchange")

        params = {
            "exchange": exchange,
        }

        url_path = "/api/v1/wallet-status/assets"
        return self.query(url_path, params)



from typing import Any
from datamaxi.api import API
from datamaxi.datamaxi.cex_candle import CexCandle
from datamaxi.datamaxi.cex_ticker import CexTicker
from datamaxi.datamaxi.cex_fee import CexFee
from datamaxi.datamaxi.cex_wallet_status import CexWalletStatus
from datamaxi.datamaxi.cex_announcement import CexAnnouncement
from datamaxi.datamaxi.cex_token import CexToken


class Cex(API):
    """Client to fetch CEX data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize CEX client.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        super().__init__(api_key, **kwargs)

        self.candle = CexCandle(api_key, **kwargs)
        self.ticker = CexTicker(api_key, **kwargs)
        self.fee = CexFee(api_key, **kwargs)
        self.wallet_status = CexWalletStatus(api_key, **kwargs)
        self.announcement = CexAnnouncement(api_key, **kwargs)
        self.token = CexToken(api_key, **kwargs)



from typing import Any
from datamaxi.lib.constants import BASE_URL
from datamaxi.datamaxi.cex import Cex
from datamaxi.datamaxi.dex import Dex
from datamaxi.datamaxi.funding_rate import FundingRate
from datamaxi.datamaxi.forex import Forex
from datamaxi.datamaxi.premium import Premium
from datamaxi.datamaxi.cex_candle import CexCandle  # used in documentation # noqa:F401
from datamaxi.datamaxi.cex_ticker import (  # used in documentation # noqa:F401
    CexTicker,
)
from datamaxi.datamaxi.cex_fee import (  # used in documentation # noqa:F401
    CexFee,
)
from datamaxi.datamaxi.cex_wallet_status import (  # used in documentation # noqa:F401
    CexWalletStatus,
)
from datamaxi.datamaxi.cex_announcement import (  # used in documentation # noqa:F401
    CexAnnouncement,
)
from datamaxi.datamaxi.cex_token import (  # used in documentation # noqa:F401
    CexToken,
)


class Datamaxi:
    """Client to fetch unified data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize the object.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        if "base_url" not in kwargs:
            kwargs["base_url"] = BASE_URL

        self.cex = Cex(api_key, **kwargs)
        self.dex = Dex(api_key, **kwargs)
        self.funding_rate = FundingRate(api_key, **kwargs)
        self.forex = Forex(api_key, **kwargs)
        self.premium = Premium(api_key, **kwargs)



import os
import logging
from datamaxi import Datamaxi


logging.basicConfig(level=logging.INFO)

api_key = os.getenv("API_KEY")
base_url = os.getenv("BASE_URL") or "https://api.datamaxiplus.com"

datamaxi = Datamaxi(api_key=api_key, base_url=base_url)


def test_cex_candle():
    datamaxi.cex.candle(
        exchange="binance",
        symbol="BTC-USDT",
        interval="1m",
        market="spot",
    )
    datamaxi.cex.candle.exchanges(market="spot")
    datamaxi.cex.candle.symbols(exchange="binance", market="spot")
    datamaxi.cex.candle.intervals()


def test_cex_ticker():
    datamaxi.cex.ticker.exchanges(market="spot")
    datamaxi.cex.ticker.symbols(exchange="binance", market="spot")


def test_cex_token_updates():
    datamaxi.cex.token.updates()


def test_cex_fees():
    datamaxi.cex.fee(exchange="binance", symbol="BTC-USDT")
    datamaxi.cex.fee.exchanges()
    datamaxi.cex.fee.symbols(exchange="binance")


def test_cex_announcement():
    datamaxi.cex.announcement()


def test_cex_wallet_status():
    datamaxi.cex.wallet_status(exchange="binance", asset="BTC")
    datamaxi.cex.wallet_status.exchanges()
    datamaxi.cex.wallet_status.assets(exchange="binance")


def test_test_funding_rate():
    datamaxi.funding_rate.history(exchange="binance", symbol="BTC-USDT")
    datamaxi.funding_rate.latest(exchange="binance", symbol="BTC-USDT")
    datamaxi.funding_rate.exchanges()
    datamaxi.funding_rate.symbols(exchange="binance")


def test_dex():
    datamaxi.dex.chains()
    datamaxi.dex.exchanges()
    datamaxi.dex.pools(exchange="klayswap", chain="kaia_mainnet")
    datamaxi.dex.intervals()
    datamaxi.dex.trade(
        exchange="pancakeswap",
        chain="bsc_mainnet",
        pool="0xb24cd29e32FaCDDf9e73831d5cD1FFcd1e535423",
    )
    datamaxi.dex.liquidity(
        exchange="pancakeswap",
        chain="bsc_mainnet",
        pool="0xb24cd29e32FaCDDf9e73831d5cD1FFcd1e535423",
    )
    datamaxi.dex.trade(
        exchange="pancakeswap",
        chain="bsc_mainnet",
        pool="0xb24cd29e32FaCDDf9e73831d5cD1FFcd1e535423",
    )


def test_forex():
    datamaxi.forex.symbols()
    datamaxi.forex(symbol="USD-KRW")


def test_premium():
    datamaxi.premium()
    datamaxi.premium.exchanges()



from typing import Any, Callable, Tuple, List, Dict, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameter
from datamaxi.lib.utils import check_required_parameters
from datamaxi.datamaxi.utils import convert_data_to_data_frame
from datamaxi.lib.constants import SPOT, FUTURES, INTERVAL_1D, USD


class CexCandle(API):
    """Client to fetch CEX candle data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize cex candle client.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        super().__init__(api_key, **kwargs)

        self.__module__ = __name__
        self.__qualname__ = self.__class__.__qualname__

    def __call__(
        self,
        exchange: str,
        market: str,
        symbol: str,
        interval: str = INTERVAL_1D,
        from_unix: str = None,
        to_unix: str = None,
        currency: str = USD,
        pandas: bool = True,
    ) -> Union[Tuple[Dict, Callable], Tuple[pd.DataFrame, Callable]]:
        """Fetch candle data

        `GET /api/v1/cex/candle`

        <https://docs.datamaxiplus.com/rest/cex/candle/data>

        Args:
            exchange (str): Exchange name
            market (str): Market type (spot/futures)
            symbol (str): Symbol name
            interval (str): Candle interval
            from_unix (str): Start time in Unix timestamp
            to_unix (str): End time in Unix timestamp
            currency (str): Currency
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Candle data in pandas DataFrame and next request function
        """
        check_required_parameters(
            [
                [exchange, "exchange"],
                [symbol, "symbol"],
                [interval, "interval"],
                [market, "market"],
                [currency, "currency"],
            ]
        )

        if market not in [SPOT, FUTURES]:
            raise ValueError("market must be either spot or futures")

        params = {
            "exchange": exchange,
            "market": market,
            "symbol": symbol,
            "interval": interval,
            "from": from_unix,
            "to": to_unix,
            "currency": currency,
        }

        res = self.query("/api/v1/cex/candle", params)
        if res["data"] is None or len(res["data"]) == 0:
            raise ValueError("no data found")

        if pandas:
            return convert_data_to_data_frame(res["data"])
        else:
            return res

    def exchanges(self, market: str) -> List[str]:
        """Fetch supported exchanges accepted by
        [datamaxi.CexCandle.get](./#datamaxi.datamaxi.CexCandle.get)
        API.

        `GET /api/v1/cex/candle/exchanges`

        <https://docs.datamaxiplus.com/rest/cex/candle/exchanges>

        Args:
            market (str): Market type (spot/futures)

        Returns:
            List of supported exchanges
        """
        check_required_parameter(market, "market")

        if market not in [SPOT, FUTURES]:
            raise ValueError("market must be either spot or futures")

        params = {"market": market}
        url_path = "/api/v1/cex/candle/exchanges"
        return self.query(url_path, params)

    def symbols(self, exchange: str = None, market: str = None) -> List[Dict]:
        """Fetch supported symbols accepted by
        [datamaxi.CexCandle.get](./#datamaxi.datamaxi.CexCandle.get)
        API.

        `GET /api/v1/cex/candle/symbols`

        <https://docs.datamaxiplus.com/rest/cex/candle/symbols>

        Args:
            exchange (str): Exchange name
            market (str): Market type (spot/futures)

        Returns:
            List of supported symbols
        """
        if market is not None and market not in [SPOT, FUTURES]:
            raise ValueError("market must be either spot or futures")

        params = {}
        if exchange is not None:
            params["exchange"] = exchange
        if market is not None:
            params["market"] = market

        url_path = "/api/v1/cex/candle/symbols"
        return self.query(url_path, params)

    def intervals(self) -> List[str]:
        """Fetch supported intervals accepted by
        [datamaxi.CexCandle.get](./#datamaxi.datamaxi.CexCandle.get)
        API.

        `GET /api/v1/candle/intervals`

        <https://docs.datamaxiplus.com/rest/cex/candle/intervals>

        Returns:
            List of supported intervals
        """
        url_path = "/api/v1/cex/candle/intervals"
        return self.query(url_path)



from typing import Any, Callable, Tuple, List, Dict, Union
import pandas as pd
from datamaxi.api import API
from datamaxi.lib.utils import check_required_parameter
from datamaxi.lib.utils import check_required_parameters
from datamaxi.datamaxi.utils import convert_data_to_data_frame
from datamaxi.lib.constants import ASC, DESC


class FundingRate(API):
    """Client to fetch funding rate data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize funding rate client.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        super().__init__(api_key, **kwargs)

    def history(
        self,
        exchange: str,
        symbol: str,
        page: int = 1,
        limit: int = 1000,
        fromDateTime: str = None,
        toDateTime: str = None,
        sort: str = DESC,
        pandas: bool = True,
    ) -> Union[Tuple[Dict, Callable], Tuple[pd.DataFrame, Callable]]:
        """Fetch historical funding rate data

        `GET /api/v1/funding-rate`

        <https://docs.datamaxiplus.com/rest/funding-rate/historical-funding-rate>

        Args:
            exchange (str): Exchange name
            symbol (str): Symbol name
            page (int): Page number
            limit (int): Limit of data
            fromDateTime (str): Start date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            toDateTime (str): End date and time (accepts format "2006-01-02 15:04:05" or "2006-01-02")
            sort (str): Sort order
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Historical funding rate data in pandas DataFrame and next request function
        """
        check_required_parameters(
            [
                [exchange, "exchange"],
                [symbol, "symbol"],
            ]
        )

        if page < 1:
            raise ValueError("page must be greater than 0")

        if limit < 1:
            raise ValueError("limit must be greater than 0")

        if fromDateTime is not None and toDateTime is not None:
            raise ValueError(
                "fromDateTime and toDateTime cannot be set at the same time"
            )

        if sort not in [ASC, DESC]:
            raise ValueError("sort must be either asc or desc")

        params = {
            "exchange": exchange,
            "symbol": symbol,
            "page": page,
            "limit": limit,
            "from": fromDateTime,
            "to": toDateTime,
            "sort": sort,
        }

        res = self.query("/api/v1/funding-rate/history", params)
        if res["data"] is None or len(res["data"]) == 0:
            raise ValueError("no data found")

        def next_request():
            return self.get(
                exchange,
                symbol,
                page + 1,
                limit,
                fromDateTime,
                toDateTime,
                sort,
                pandas,
            )

        if pandas:
            df = convert_data_to_data_frame(res["data"])
            return df, next_request
        else:
            return res, next_request

    def latest(
        self,
        sort: str = None,
        limit: int = None,
        symbol: str = None,
        exchange: str = None,
        pandas: bool = True,
    ) -> Union[Tuple[List, Callable], Tuple[pd.DataFrame, Callable]]:
        """Fetch latest funding rate data

        `GET /api/v1/funding-rate/latest`

        <https://docs.datamaxiplus.com/rest/funding-rate/latest-funding-rate>

        Args:
            sort (str): Sort data by `asc` or `desc`
            limit (int): Limit number of data to return
            symbol (str): Symbol name
            exchange (str): exchange name
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Latest funding rate data in pandas DataFrame
        """
        params = {}

        if sort is not None:
            params["sort"] = sort

        if limit is not None:
            params["limit"] = limit

        if symbol is not None:
            params["symbol"] = symbol

        if exchange is not None:
            params["exchange"] = exchange

        res = self.query("/api/v1/funding-rate/latest", params)

        if pandas:
            df = pd.DataFrame([res])
            df = df.set_index("d")
            return df
        else:
            return res

    def exchanges(self) -> List[str]:
        """Fetch supported exchanges accepted by
        [datamaxi.FundingRate.get](./#datamaxi.datamaxi.FundingRate.get)
        API.

        `GET /api/v1/funding-rate/exchanges`

        <https://docs.datamaxiplus.com/rest/funding-rate/exchanges>

        Returns:
            List of supported exchanges
        """
        url_path = "/api/v1/funding-rate/exchanges"
        return self.query(url_path)

    def symbols(self, exchange: str, market: str = "spot") -> List[str]:
        """Fetch supported symbols accepted by
        [datamaxi.FundingRate.get](./#datamaxi.datamaxi.FundingRate.get)
        API.

        `GET /api/v1/funding-rate/symbols`

        <https://docs.datamaxiplus.com/rest/funding-rate/symbols>

        Args:
            exchange (str): Exchange name

        Returns:
            List of supported symbols
        """
        check_required_parameter(exchange, "exchange")
        params = {"exchange": exchange}
        url_path = "/api/v1/funding-rate/symbols"
        return self.query(url_path, params)



from typing import Any, List, Union
import pandas as pd
from datamaxi.api import API


class Premium(API):
    """Client to fetch premium data from DataMaxi+ API."""

    def __init__(self, api_key=None, **kwargs: Any):
        """Initialize premium client.

        Args:
            api_key (str): The DataMaxi+ API key
            **kwargs: Keyword arguments used by `datamaxi.api.API`.
        """
        super().__init__(api_key, **kwargs)

        self.__module__ = __name__
        self.__qualname__ = self.__class__.__qualname__

    def __call__(  # noqa: C901
        self,
        source_exchange: str = None,
        target_exchange: str = None,
        asset: str = None,
        source_quote: str = None,
        target_quote: str = None,
        sort: str = None,
        key: str = None,
        page: int = 1,
        limit: int = 100,
        currency: str = None,
        pandas: bool = True,
    ) -> Union[List, pd.DataFrame]:
        """Fetch premium data

        `GET /api/v1/premium`
        <https://docs.datamaxiplus.com/rest/premium/premium>

        Args:
            source_exchange (str): Source exchange name
            target_exchange (str): Target exchange name
            asset (str): Asset name
            source_quote (str): Source quote currency
            target_quote (str): Target quote currency
            currency (str): Currency applied to cross-exchange price differences
            sort (str): Sort data by `asc` or `desc`
            key (str): Key to sort data
            page (int): Page number
            limit (int): Page size
            pandas (bool): Return data as pandas DataFrame

        Returns:
            Premium data in pandas DataFrame
        """
        params = {}

        if source_exchange is not None:
            params["sourceExchange"] = source_exchange

        if target_exchange is not None:
            params["targetExchange"] = target_exchange

        if asset is not None:
            params["asset"] = asset

        if source_quote is not None:
            params["sourceQuote"] = source_quote

        if target_quote is not None:
            params["targetQuote"] = target_quote

        if sort is not None:
            params["sort"] = sort

        if page is not None:
            params["page"] = page

        if key is not None:
            params["key"] = key

        if limit is not None:
            params["limit"] = limit

        if currency is not None:
            params["currency"] = currency

        res = self.query("/api/v1/premium", params)
        if res["data"] is None or len(res["data"]) == 0:
            raise ValueError("no data found")

        if pandas:
            df = pd.DataFrame(res["data"])
            df = df.set_index("d")
            return df
        else:
            return res

    def exchanges(self) -> List[str]:
        """Fetch supported exchanges accepted by
        [datamaxi.Premium.get](./#datamaxi.datamaxi.Premium.get)
        API.

        `GET /api/v1/Premium/exchanges`

        <https://docs.datamaxiplus.com/rest/Premium/exchanges>

        Returns:
            List of supported exchange
        """
        url_path = "/api/v1/premium/exchanges"
        return self.query(url_path)
